MODULE Zuege EXPORTS Main;

(* Dieses Programm realisiert Zuege mit einer Lokomotive und mehreren
   Wagen als einfach verkettete Liste.

   Autor    : Moritz Schnizler, RWTH Aachen
   Umgebung : PM 3, Windows NT 4.0
   Erstellt : 04.12.00  Letzte Aenderung: 04.12.00 *)


IMPORT SIO; (* Importiere Operationen fuer die Ein-/Ausgabe *)


TYPE LokKennung  = TEXT;       (* Beliebiger Text als Lokkennung *)
     Wagennummer = [1 .. 999]; (* Eindeutige Wagennummer *)


     (* Datenstruktur fuer einen Wagen *)

     WagenRef = REF Wagen;

     Wagen    = RECORD
                  wagennr        : Wagennummer;
                  naechsterWagen : WagenRef;
                END;


     (* Datentyp fuer ganzen Zug (Lok) *)

     Zug = RECORD
             lokName : LokKennung;
             wagen   : WagenRef;
           END;


PROCEDURE BereitMachen(VAR zug: Zug; kennung: TEXT) =
(* Kennung des Zuges setzen und sicher stellen, dass noch keine Wagen angehaengt sind. *)
BEGIN
  zug.lokName := kennung;
  zug.wagen := NIL;
END BereitMachen;


PROCEDURE Anhaengen(VAR zug: Zug; wagen: WagenRef) =
(* Haengt den gegebenen Wagen ans Ende des uebergebenen Zuges *)
VAR aktWagen: WagenRef;

BEGIN
  (* Wagen wird letzter Wagen des Zuges, also keine Nachfolger *)
  wagen^.naechsterWagen := NIL;

  (* Ende des Zuges suchen und Wagen anhaengen *)
  IF zug.wagen = NIL THEN
    (* Keine Wagen angehaengt, also an Lok anhaengen *)
    zug.wagen := wagen;
  ELSE
    (* Letzten Wagen suchen und dort anhaengen *)
    aktWagen := zug.wagen;
    WHILE aktWagen^.naechsterWagen # NIL DO
      aktWagen := aktWagen^.naechsterWagen;
    END;
    aktWagen^.naechsterWagen := wagen;
  END;
END Anhaengen;


PROCEDURE Ausgeben (zug : Zug)=
(* Gibt kompletten Zug von Lok bis letztem Wagen aus *)
VAR aktWagen : WagenRef;

BEGIN
  SIO.PutText(zug.lokName);

  (* Alle Wagen von vorn nach hinten ausgeben *)
  aktWagen := zug.wagen;
  WHILE aktWagen # NIL DO
    SIO.PutText(" - "); SIO.PutInt(aktWagen^.wagennr);
    aktWagen := aktWagen^.naechsterWagen;
  END;
END Ausgeben;


PROCEDURE Umkehren (VAR zug1, zug2 : Zug) =
(* Haengt die Wagen des ersten Zuges umgekehrt an den zweiten Zug *)
VAR aktWagen : WagenRef;

BEGIN
  WHILE zug1.wagen # NIL DO
    aktWagen := zug1.wagen;
    zug1.wagen := aktWagen^.naechsterWagen;
    aktWagen^.naechsterWagen := zug2.wagen;
    zug2.wagen := aktWagen;
  END;
END Umkehren;

VAR fruehzug, spaetzug: Zug;
    wagen             : WagenRef;

BEGIN
  (* Fruehzug zusammenstellen *)
  BereitMachen(fruehzug, "ICE 205");
 
  (* Neuen Wagen erzeugen und anhaengen *)
  wagen := NEW(WagenRef);
  wagen^.wagennr := 101;
  wagen^.naechsterWagen := NIL;
  Anhaengen(fruehzug, wagen);

  (* Neuen Wagen erzeugen und anhaengen *)
  wagen := NEW(WagenRef, wagennr := 107, naechsterWagen := NIL);
  Anhaengen(fruehzug, wagen);

  (* Noch drei Wagen erzeugen und anhaengen *)
  wagen := NEW(WagenRef, wagennr := 103, naechsterWagen := NIL);
  Anhaengen(fruehzug, wagen);
  wagen := NEW(WagenRef, wagennr := 97, naechsterWagen := NIL);
  Anhaengen(fruehzug, wagen);
  wagen := NEW(WagenRef, wagennr := 115, naechsterWagen := NIL);
  Anhaengen(fruehzug, wagen);

  (* Fruehzug ausgeben *)
  SIO.PutText("Fruehzug: ");
  Ausgeben(fruehzug); SIO.Nl();

  (* Zeit fuer den Spaetzug *)
  BereitMachen(spaetzug, "ICE 207");
  (* Spaetzug ausgeben *)
  SIO.PutText("Spaetzug: ");
  Ausgeben(spaetzug); SIO.Nl();

  (* Wagen des Fruehzugs umgekehrt an Spaetzug haengen *)
  Umkehren(fruehzug, spaetzug);
  SIO.PutText("Fruehzug: "); Ausgeben(fruehzug); SIO.Nl();
  SIO.PutText("Spaetzug: "); Ausgeben(spaetzug); SIO.Nl();

END Zuege.



  
