MODULE Instrument;

(* Das Objektmodul Instrument ist eine billige Moeglichkeit, um die
   Zweige eines Programms zu instrumentieren. Allerdings ist die hier
   gewaehlte Loesung auf maximal 99 verwaltbare Zweige begrenzt.

   Autor           : Moritz Schnizler, RWTH Aachen
   Umgebung        : PM 3, Windows 2000
   Erstellt        : 19.12.00
   Letzte Aenderung: 20.12.00 *)

IMPORT SIO; (* Importiere Operationen fuer Ein-/Ausgabe *)

(* Gekapselte Datenstruktur *)

(* Zaehler fuer Programmzweige mit Maximum der belegten Zaehler *) 

VAR zaehler := ARRAY [1..MaxZweige] OF CARDINAL {0, ..};
    maximum : [0..MaxZweige];


PROCEDURE Initialisiere(zweige: [1..MaxZweige]) =
(* Initialisiert das Objektmodul mit der Anzahl der Programmzweige *)
BEGIN
  maximum := zweige;
END Initialisiere;


PROCEDURE ZaehleZweig(i: CARDINAL) =
(* Erhoeht den Zaehler fuer Programmzweig i um eins *)
BEGIN
  IF i >= FIRST(zaehler) AND i <= maximum THEN
    INC(zaehler[i]);
  ELSE
    SIO.PutLine("Fehler im Modul Instrument: Dieser Zweig existiert nicht!")
  END;
END ZaehleZweig; 


PROCEDURE ZahlAusgeben(wert: CARDINAL) =
(* Hilfsprozedur gibt den Wert immer zweistellig aus *)
BEGIN
  IF wert < 10 THEN SIO.PutChar('0'); END;
  SIO.PutInt(wert);
END ZahlAusgeben;


PROCEDURE ZaehlerAusgeben() =
(* Gibt die ermittelten Zaehlerwerte auf dem Bildschirm aus *)
BEGIN
  (* Alle Zaehler ausgeben *)
  FOR i := FIRST(zaehler) TO maximum DO
    (* Bezeichung des Zaehlers ausgeben *)
    SIO.PutText("Z");
    ZahlAusgeben(i);
    SIO.PutText(": ");

    (* Wert des Zaehlers ausgeben *)
    ZahlAusgeben(zaehler[i]);

    IF i MOD 8 = 0 THEN
      (* Nach acht Zaehlern neue Zeile anfangen! *)
      SIO.Nl();  
    ELSE
      (* Sonst nur zwei Leerzeichen einfuegen *)
      SIO.PutText("  ");
    END;
  END;
END ZaehlerAusgeben;


PROCEDURE Ueberdeckungsgrad(): REAL =
(* Gibt den Grad der Zweigueberdeckung in Prozent zurueck *)
VAR anzahl: CARDINAL := 0;

BEGIN
  (* Bestimme Anzahl der Zaehler groesser 0 *)
  FOR i := FIRST(zaehler) TO maximum DO
    IF zaehler[i] > 0 THEN anzahl := anzahl + 1; END;
  END;

  RETURN (FLOAT(anzahl, REAL) / FLOAT(maximum, REAL)) * 100.0;
END Ueberdeckungsgrad;


BEGIN
  maximum := 0; (* Zunaechst werden keine Zweige verwaltet *)
END Instrument.