MODULE Diskographie;

(* Dieses Module implementiert die Diskographie-Verwaltung, wie in der Schnittstelle
   Diskographie definiert. Die Verwaltung erfolgt mit Hilfe eines Feldes begrenzter Kapazitaet.
   Autor           : Thomas von der Maen, RWTH Aachen
   Umgebung        : PM-3 Windows 2000
   Erstellt        : 19.12.00  Letzte Aenderung: 19.12.00 
*)

IMPORT SIO;
IMPORT CD;   (* Importiere den ADT CD *)

CONST Max = 5;

TYPE DiskographieIndex = [0 .. Max];
     Diskographieverwaltung = ARRAY [1 .. Max] OF CD.T;

VAR index : DiskographieIndex;
    verwaltung: Diskographieverwaltung;


PROCEDURE Init()=
(* Initialisiert die Diskographie. Diese enthlt dann keine CDs *)
BEGIN
  index := 0;
END Init;


PROCEDURE AddCD(cd: CD.T)=
(* Fgt die bergebene CD der Diskographie hinzu *)
BEGIN
  (* CD wird im nchsten Speicherplatz abgelegt *)
  IF NOT IsFull() THEN
    index := index + 1;
    verwaltung[index] := cd;
  ELSE
    SIO.PutLine("Diskographie ist voll!!!");
  END;
END AddCD;


PROCEDURE RemoveCD(pos: CARDINAL)=
(* Lscht die CD an der bergebenen Indexposition *)
BEGIN
  IF IsEmpty() THEN
    SIO.PutLine("Keine CD zum loeschen vorhanden!!!");
  ELSE
    IF (pos > 0) AND (pos <= index) THEN 
      FOR i:=pos TO index - 1 DO
        verwaltung[i] := verwaltung[i+1];
      END;
      (* Indexposition wird um 1 zurckgesetzt *)
      index := index - 1;
    END;
  END;
END RemoveCD;


PROCEDURE IsFull(): BOOLEAN =
(* Prft, ob die Diskographie voll ist *)
BEGIN
  RETURN (index = Max);
END IsFull;


PROCEDURE IsEmpty(): BOOLEAN =
(* Prft, ob die Diskographie leer ist *)
BEGIN
  RETURN (index = 0);
END IsEmpty;


PROCEDURE Print()=
(* Listet den Inhalt der Diskographie auf dem Bildschirm *)
BEGIN
  SIO.PutLine("-------------------- Diskographieverwaltung ----------------------------------------------");
  SIO.Nl();
  FOR i:=1 TO index DO
    SIO.PutInt(i); SIO.PutText(".   ");
    SIO.PutText("Titel: "); SIO.PutText(CD.GetTitle(verwaltung[i]));
    SIO.PutText("   Interpret: "); SIO.PutText(CD.GetInterpret(verwaltung[i]));
    SIO.PutText("   Anzahl Tracks: "); SIO.PutInt(CD.GetTrackCount(verwaltung[i]));
    SIO.PutText("   Gesamtspieldauer: "); SIO.PutInt(CD.GetTotalTime(verwaltung[i]));
    SIO.Nl();
  END;  
  SIO.PutLine("------------------------------------------------------------------------------------------");
END Print;


PROCEDURE PrintWithTracklist()=
(* Listet den Inhalt der Diskographie mit den Tracks jeder CD auf dem Bildschirm *)
BEGIN
  SIO.PutLine("-------------------- Diskographieverwaltung ----------------------------------------------");
  SIO.Nl();
  FOR i:=1 TO index DO
    SIO.PutInt(i); SIO.PutText(".   ");
    SIO.PutText("Titel: "); SIO.PutText(CD.GetTitle(verwaltung[i]));
    SIO.PutText("   Interpret: "); SIO.PutText(CD.GetInterpret(verwaltung[i]));
    SIO.PutText("   Anzahl Tracks: "); SIO.PutInt(CD.GetTrackCount(verwaltung[i]));
    SIO.PutText("   Gesamtspieldauer: "); SIO.PutInt(CD.GetTotalTime(verwaltung[i]));
    SIO.Nl();
    SIO.PutLine("Tracks:");
    CD.PrintTracklist(verwaltung[i]);
    SIO.Nl();
  END;
  SIO.PutLine("------------------------------------------------------------------------------------------");
END PrintWithTracklist;

PROCEDURE GetCD(i: CARDINAL): CD.T =
(* Liefert die CD der Diskographie mit dem bergebenen Index *)
BEGIN
  IF (i > 0) AND (i <= index) THEN
    RETURN (verwaltung[i]);
  ELSE
    SIO.PutLine("Keine gueltige CD angegeben");
    RETURN NIL;
  END;
END GetCD;

BEGIN
END Diskographie.