MODULE Diskographie;

(* Modul zur Klasse Diskographie
   Autor           : Antje Nowack, Thomas van der Massen
   Umgebung        : PM-3 Windows 95
   Erstellt        : 19.12.00  Letzte Aenderung: 22.01.01 
*)

IMPORT SIO;
IMPORT CD;   (* Importiere den ADT CD *)
IMPORT Tontraeger;
IMPORT Schallplatte;
IMPORT Kassette;


CONST Max = 5;

REVEAL
      T = Public BRANDED OBJECT
                  verwaltung:  ARRAY [1 .. Max] OF Tontraeger.T;
                  typ : ARRAY [1..Max] OF TontraegerTypen;
                  index : CARDINAL := 0;
            OVERRIDES
                AddCD := AddCDDisk;
                (* Fgt die bergebene CD der Diskographie hinzu *)

                AddSchallplatte := AddSchallplatteDisk;
                (* Fgt die bergebene CD der Diskographie hinzu *)

                AddKassette := AddKassetteDisk;
                (* Fgt die bergebene CD der Diskographie hinzu *)

                GetTt := GetTtDisk;
                (* Liefert die CD der Diskographie mit dem bergebenen Index *)

                RemoveTt:= RemoveTtDisk;
                (* Lscht die CD an der bergebenen Indexposition *)

                IsFull := IsFullDisk;
                (* Prft, ob die Diskographie voll ist *)

                IsEmpty := IsEmptyDisk;
                (* Prft, ob die Diskographie leer ist *)

                Print := PrintDisk;
                (* Listet den Inhalt der Diskographie auf dem Bildschirm *)
  
                PrintWithTracklist := PrintWithTracklistDisk; 
                (* Listet den Inhalt der Diskographie inklusive der Trackliste jeder CD auf dem Bildschirm *)

               TypAnStelle := TypAnStelleDisk;
               (* Liefert den Typ des Tontraegers an der i-ten Stelle *)

               TontraegerAnStelle := TontraegerAnStelleDisk;
               (* Liefert den Tontraeger an der i-ten Stelle. *)
 
         END;



PROCEDURE AddCDDisk(self: T;  cd: CD.T)=
(* Fgt die bergebene CD der Diskographie hinzu *)
BEGIN
  (* CD wird im nchsten Speicherplatz abgelegt *)
  IF NOT self.IsFull() THEN
    self.index := self.index + 1;
    self.verwaltung[self.index] := cd;
    self.typ[self.index] := TontraegerTypen.CDTyp;
  ELSE
    SIO.PutLine("Diskographie ist voll!!!");
  END;
END AddCDDisk;


PROCEDURE AddSchallplatteDisk(self: T;  sch: Schallplatte.T)=
(* Fgt die bergebene CD der Diskographie hinzu *)
BEGIN
  (* CD wird im nchsten Speicherplatz abgelegt *)
  IF NOT self.IsFull() THEN
    self.index := self.index + 1;
    self.verwaltung[self.index] := sch;
    self.typ[self.index] := TontraegerTypen.SchallplatteTyp;
  ELSE
    SIO.PutLine("Diskographie ist voll!!!");
  END;
END AddSchallplatteDisk;

PROCEDURE AddKassetteDisk(self: T;  kass: Kassette.T)=
(* Fgt die bergebene CD der Diskographie hinzu *)
BEGIN
  (* CD wird im nchsten Speicherplatz abgelegt *)
  IF NOT self.IsFull() THEN
    self.index := self.index + 1;
    self.verwaltung[self.index] := kass;
    self.typ[self.index] := TontraegerTypen.KassetteTyp;
  ELSE
    SIO.PutLine("Diskographie ist voll!!!");
  END;
END AddKassetteDisk;


PROCEDURE RemoveTtDisk(self:T; pos: CARDINAL)=
(* Lscht die CD an der bergebenen Indexposition *)
BEGIN
  IF self.IsEmpty() THEN
    SIO.PutLine("Kein Tontraeger zum Loeschen vorhanden!!!");
  ELSE
    IF (pos > 0) AND (pos <=self. index) THEN 
      FOR i:=pos TO self.index - 1 DO
        self.verwaltung[i] := self.verwaltung[i+1];
        self.typ[i] := self.typ[i+1];
      END;
      (* Indexposition wird um 1 zurckgesetzt *)
      self.index := self.index - 1;
    END;
  END;
END RemoveTtDisk;


PROCEDURE IsFullDisk(self: T): BOOLEAN =
(* Prft, ob die Diskographie voll ist *)
BEGIN
  RETURN (self.index = Max);
END IsFullDisk;


PROCEDURE IsEmptyDisk(self:T): BOOLEAN =
(* Prft, ob die Diskographie leer ist *)
BEGIN
  RETURN (self.index = 0);
END IsEmptyDisk;


PROCEDURE PrintDisk(self : T)=
(* Listet den Inhalt der Diskographie auf dem Bildschirm *)
BEGIN
  SIO.PutLine(" *** Diskographieverwaltung *** ");
  SIO.Nl();
  FOR i:=1 TO self.index DO
    SIO.PutInt(i); SIO.PutText(".   ");
    SIO.PutText("Titel: ");
    SIO.PutText((self.verwaltung[i]).GetTitle());
    SIO.PutText("   Interpret: ");
    SIO.PutText((self.verwaltung[i]).GetInterpret());
    SIO.PutText("   Anzahl Tracks: ");
    SIO.PutInt((self.verwaltung[i]).GetTrackCount()); 
    SIO.Nl();
  END;  
END PrintDisk;


PROCEDURE PrintWithTracklistDisk(self:T)=
(* Listet den Inhalt der Diskographie mit den Tracks jeder CD auf dem Bildschirm *)
BEGIN
  SIO.PutLine(" *** Diskographieverwaltung *** ");
  SIO.Nl();
  FOR i:=1 TO self.index DO
    SIO.PutInt(i); SIO.PutText(".   ");
    IF self.typ[i] = TontraegerTypen.CDTyp 
    THEN SIO.PutLine(CD.T.PrintInfo(self.verwaltung[i])) END;
    IF self.typ[i] = TontraegerTypen.SchallplatteTyp 
    THEN SIO.PutLine(Schallplatte.T.PrintInfo(self.verwaltung[i])) END;
    IF self.typ[i] = TontraegerTypen.KassetteTyp 
    THEN SIO.PutLine(Kassette.T.PrintInfo(self.verwaltung[i])) END;
  END; 
END PrintWithTracklistDisk;

PROCEDURE GetTtDisk(self: T; i: CARDINAL): Tontraeger.T =
(* Liefert die CD der Diskographie mit dem bergebenen Index *)
BEGIN
  IF (i > 0) AND (i <= self.index) THEN
    RETURN (self.verwaltung[i]);
  ELSE
    SIO.PutLine("Keine gueltige CD angegeben");
    RETURN NIL;
  END;
END GetTtDisk;

PROCEDURE TypAnStelleDisk(self : T; i : CARDINAL) : TontraegerTypen =
(* Liefert den Typ des Tontraegers an der i-ten Stelle *)
BEGIN
  RETURN self.typ[i];
END TypAnStelleDisk;

PROCEDURE TontraegerAnStelleDisk(self: T; i: CARDINAL): Tontraeger.T =
(* Liefert den Tontraeger an der i-ten Stelle. *)
BEGIN
  RETURN self.verwaltung[i];
END TontraegerAnStelleDisk;
   

BEGIN
END Diskographie.
