MODULE Schuelerliste EXPORTS Main;

(* Autor: Antje Nowack
   Umgebung: PM3, Windows95
   Erstellt: 16. 12. 2000
   Letzte Aenderung: 21. 12. 2000
*)

IMPORT SIO, Math;

(*---------------------------Aufgabenteil a -------------------------------*)

(* Ein Schueler wird durch einen Verbund/Record mit den entsprechenden
   Datentypen realisiert. *)
TYPE Schueler = RECORD
                  Name : TEXT;
                  Vorname : TEXT;
                  Alter : REAL;
                  Note : REAL;
                END;

     (* Mit Hilfe der Datenstruktur SchuelerRef laesst sich eine einfach
        verkettete Liste, in der Schueler abgelegt sind, realisieren. *)
     SchuelerRef = REF Element;
     Element = RECORD
                  schueler : Schueler;        
                  naechster : SchuelerRef;
               END;

     (* Die folgenden beiden Typen sind Prozedurtypen. 
        Operation enthaelt alle zweistelligen Funktionen auf REAL, welche
        einen REAL-Wert zurueckliefern. Dies sind z.B. Addition und
        Multiplikation. *)
     Operation = PROCEDURE (a,b : REAL) : REAL;

     (* Projektion enthaelt alle einstelligen Funktionen, die Schueler auf
        REAL abbilden. Dies sind z.B. Die Funktionen, die einen Schueler auf
        seine Note oder sein Alter abbilden. *)
     Projektion = PROCEDURE (schueler : Schueler) : REAL;

     (* Vergleich enthaelt alle zweistelligen Relationen, d.h. booleschen Funktionen,
        auf reellen Zahlen, z.B. die <- und die >-Relation. *)
     Vergleich = PROCEDURE (a,b : REAL) : BOOLEAN;

(* ... einige Hilfsvariablen ... *)
VAR schuelerListe : SchuelerRef := NIL;
    schueler : Schueler;
    wahl: INTEGER;
    dummy : TEXT; 
    sortiert : SchuelerRef;
    limitAlter, limitNote : REAL;


PROCEDURE Einfuegen(VAR liste : SchuelerRef; neu : Schueler)=
(* Diese Prozedur fuegt einen Schueler in eine Liste ein. Dieser wird
   an den Beginn der Liste gesetzt. Hierbei wird nicht ueberpruft, ob der
   Schueler bereits in der Liste vorhanden ist. *)

VAR elem : SchuelerRef;
BEGIN
  elem := NEW(SchuelerRef);
  elem^.schueler := neu;
  elem^.naechster := liste;
  liste := elem;  
END Einfuegen;


PROCEDURE Ausgeben(liste : SchuelerRef) = 
(* Diese Prozedur gibt eine Liste auf dem Bildschirm aus, indem die
   gesamte Liste durchlaufen wird. *)

BEGIN
  IF liste # NIL THEN
    SIO.PutText("Name: ");
    SIO.PutText(liste^.schueler.Name);
    SIO.Nl();
    SIO.PutText("Vorname: ");
    SIO.PutText(liste^.schueler.Vorname);
    SIO.Nl();
    SIO.PutText("Alter: ");
    SIO.PutReal(liste^.schueler.Alter);
    SIO.Nl();
    SIO.PutText("Note: ");
    SIO.PutReal(liste^.schueler.Note);
    SIO.Nl();
    SIO.Nl();
    SIO.PutText("------------------------------------------------");
    SIO.Nl();
    dummy := SIO.GetLine(); 
    SIO.Nl();
    Ausgeben(liste^.naechster);
    SIO.Nl();
  END;
END Ausgeben;


(*---------------------------Aufgabenteil b --------------------------------*)


PROCEDURE Alter (schueler : Schueler) : REAL = 
(* Diese Funktion bildet einen Schueler auf sein Alter ab. *)

BEGIN
  RETURN schueler.Alter;
END Alter;


PROCEDURE Note (schueler : Schueler) : REAL = 
(* Die folgende Funktion bildet einen Schueler auf seine Note ab. *)

BEGIN
  RETURN schueler.Note;
END Note;

PROCEDURE A_groesser_B ( a, b : REAL) : BOOLEAN =
BEGIN
 RETURN a > b;
END A_groesser_B;  

PROCEDURE A_kleiner_B ( a, b : REAL) : BOOLEAN =
BEGIN
 RETURN a < b;
END A_kleiner_B;  

PROCEDURE StelleSuchen(liste : SchuelerRef; person : Schueler; proj : Projektion; rel : Vergleich) : SchuelerRef =
(* Mit Hilfe dieser Prozedur wird zu einem Schueler und einer
  Schuelerliste die Stelle gesucht, wo der Schueler eingefuegt werden muss.
  Hierbei wird von einer sortierten Liste gemaess rel ausgegangen. *)

VAR stelle, vergleich : SchuelerRef;
BEGIN
  vergleich := liste;
  stelle := liste;
  WHILE (vergleich # NIL) AND NOT rel(proj ( vergleich^.schueler), proj(person)) DO
    stelle := vergleich;
    vergleich := vergleich^.naechster; 
  END;
  RETURN stelle;  
END StelleSuchen;


PROCEDURE Filter( liste : SchuelerRef; proj : Projektion; rel : Vergleich; limit : REAL; VAR filterListe : SchuelerRef) =
(* Diese Prozedur liefert zu einer einfach verketteten Liste, einer
   Funktion, die einen Schueler auf ein REAL-Zahl abbildet und einer REAL-Zahl
   die Liste, die Liste der Schueler, deren Bild unter der Funktion ueber der
   Zahl liegen. Die Datensaetze der produzierten Liste sind Kopien der
   Original-Liste und diese ist nach Werten der uebergebenen Funktion
   sortiert gemaess rel. *)

VAR kopie : SchuelerRef;
BEGIN
  IF liste # NIL THEN
    (* Speicherplatz reservieren *)
    kopie := NEW(SchuelerRef);
    (* Datensatz des aktuell betrachteten Listenelementes kopieren *)
    kopie^.schueler := liste^.schueler;
    (* nur wenn die Grenze ueberschritten wurde, wird der schueler in die
       resultierende Liste aufgnommen. *)
    IF rel ( proj(kopie^.schueler), limit) THEN
        (* Die erste Operation zum Einfuegen kann nur durchgefuehrt werden,
           wenn die resultierende Liste bereits mindestens ein Element
           enthaelt und der Funktionswert muss kleiner sein als der
           einzufuegende. Dann wird das neue Element an der berechneten Stelle
           eingefuegt. *)
        IF filterListe # NIL AND  rel (proj(kopie^.schueler),proj(filterListe^.schueler)) THEN 
          kopie^.naechster := StelleSuchen(filterListe, kopie^.schueler, proj, rel)^.naechster; 
          StelleSuchen(filterListe, kopie^.schueler, proj, rel)^.naechster := kopie;
        (* Sonst wird das neue Element an den Anfang der bis jetzt berechneten
           resultierenden Liste gesetzt. *)
        ELSE
          kopie^.naechster := filterListe;
          filterListe := kopie;
        END;
    END;
    (* rekursiver Aufruf der Prozedur *)
    Filter(liste^.naechster, proj, rel, limit, filterListe);
  END;
END Filter;


PROCEDURE FilterAlter(liste: SchuelerRef; limit : REAL; VAR ergebnis : SchuelerRef) =
(* Diese Prozedur wendet die Prozedur Filter an, wobei durch die
   uebergebene Funktion jeder Schueler auf sein Alter projiziert wird und >
   als Vergleichsrelation uebergeben wird. *)

BEGIN
  Filter( liste, Alter, A_groesser_B,limit, ergebnis);
END FilterAlter;


PROCEDURE FilterNote_schlechter_spaeter(liste: SchuelerRef; limit : REAL; VAR ergebnis : SchuelerRef) =
(* Diese Prozedur wendet die Prozedur Filter an, wobei durch die
   uebergebene Funktion jeder Schueler auf seine Note projiziert wird und <
   als Vergleichsrelation uebergeben wird. *)

BEGIN
  Filter( liste, Note, A_groesser_B, limit, ergebnis);
END FilterNote_schlechter_spaeter;

PROCEDURE FilterNote_besser_spaeter(liste: SchuelerRef; limit : REAL; VAR ergebnis : SchuelerRef) =
(* Diese Prozedur wendet die Prozedur Filter an, wobei durch die
   uebergebene Funktion jeder Schueler auf seine Note projiziert wird und <
   als Vergleichsrelation uebergeben wird. *)

BEGIN
  Filter( liste, Note, A_kleiner_B, limit, ergebnis);
END FilterNote_besser_spaeter;


PROCEDURE Filtern_schlechter_spaeter(liste: SchuelerRef; limitAlter, limitNote : REAL; VAR ergebnis : SchuelerRef) =
(* Diese Prozedur wendet die beiden vorhergehenden Prozeduren an. Zuerst
   diejenige fuer die Filterung nach Alter. Auf die hieraus resultierende Liste
   wird der Filter nach Noten angewandt, welcher die Schueler liefert,
   die schlechter als eine einzugebende Grenze enthaelt.*)

VAR zwischenErgebnis : SchuelerRef;
BEGIN
  FilterAlter(liste, limitAlter, zwischenErgebnis);
  FilterNote_schlechter_spaeter(zwischenErgebnis, limitNote, ergebnis);
END Filtern_schlechter_spaeter;

PROCEDURE Filtern_besser_spaeter(liste: SchuelerRef; limitAlter, limitNote : REAL; VAR ergebnis : SchuelerRef) =
(* Diese Prozedur wendet die beiden vorhergehenden Prozeduren an. Zuerst
   diejenige fuer die Filterung nach Alter. Auf hieraus resultierende Liste
   wird der Filter nach Noten angewandt, welcher die Schueler liefert,
   die besser als eine einzugebende Grenze enthaelt.*)

VAR zwischenErgebnis : SchuelerRef;
BEGIN
  FilterAlter(liste, limitAlter, zwischenErgebnis);
  FilterNote_besser_spaeter(zwischenErgebnis, limitNote, ergebnis);
END Filtern_besser_spaeter;


(*---------------------------Aufgabenteil c -------------------------------*)


PROCEDURE Anwenden(op : Operation; liste : SchuelerRef; init : REAL) : REAL =
(* Dieser Prozedur werden eine zweistellige Funktion auf REAL-Werten
   uebergeben, eine Schuelerliste und ein REAL-Wert uebergeben. Die Funktion
   wird sukzessive auf die Noten der Liste angewandt. Der resultierende
   Wert wird zurueckgegeben. Der uebergebene REAL-Wert dient hierbei als
   Startwert.
   Die sukzessive Anwendung wird durch rekusiven Aufruf der Prozedur
   durchgefuehrt. *)

BEGIN
  IF liste = NIL THEN 
    RETURN init;
  ELSE
    init := op(init, liste^.schueler.Note); 
    RETURN Anwenden(op, liste^.naechster, init); 
  END; 
END Anwenden;


PROCEDURE Add(a,b : REAL) : REAL =
(* Addition *)

BEGIN
  RETURN (a+b);
END Add;


PROCEDURE Mult(a,b : REAL) : REAL =
(* Multiplikation *)

BEGIN
  RETURN (a*b);
END Mult;


PROCEDURE Inc(a, b : REAL) : REAL =
(* Inkrementierung des ersten Wertes *)

BEGIN
  RETURN (a + 1.0)
END Inc;


PROCEDURE Arithm_Mittel(liste : SchuelerRef) : REAL =
(* Diese Funktion liefert zu einer Schuelerliste das arithmetische
   Mittel der Noten. *)

BEGIN
  IF liste # NIL THEN
    (* Nach Definition des arithmetischen Mittels wird die Summe der Noten
       durch die Anzahl der Elemente in der Liste dividiert.
       Die Summe wird berechnet durch sukzessives Anwenden der Addition.
       Startwert ist hierbei 0.
       Die Anzahl der Elemente kann hierbei durch sukzessives Anwenden der
       Funktion Inc berechnet werden. Pro Listenelement wird um 1 erhoeht. *)
    RETURN (Anwenden(Add, liste, 0.0) / Anwenden(Inc, liste, 0.0));
  ELSE
    (* Bei der leeren Liste wird 0 zurueckgeliefert. *)
    RETURN 0.0;
  END;
END Arithm_Mittel;


PROCEDURE Geom_Mittel(liste : SchuelerRef) : REAL =
(* Diese Funktion liefert zu einer Schuelerliste das geometrische
   Mittel der Noten. *)

BEGIN
  IF liste # NIL THEN
  (* Nach Definition des geometrischen Mittels wird aus dem Produkt der Noten
     die n-te Wurzel gezogen. n ist hierbei die Anzahl der Elemente in der
     Liste.
     Das Produkt wird berechnet durch sukzessives Anwenden der Multiplikation.
     Startwert ist hierbei 1.
     Die Anzahl der Elemente kann hierbei durch sukzessives Anwenden der
     Funktion Inc berechnet werden. Pro Listenelement wird um 1 erhoeht.
     Die Verwendung von FLOAT ist notwendig, da die Potenzierungfunkt pow
     aus Math fuer LONGREAL-Werte definiert ist und Noten REAL-Werte sind. *)
     RETURN FLOAT( Math.pow ( FLOAT ( Anwenden(Mult, liste, 1.0), LONGREAL), FLOAT(1.0 / Anwenden(Inc, liste, 0.0), LONGREAL)), REAL); 
  ELSE
    RETURN 1.0 ;
  END;
END Geom_Mittel;


(*------------------------- Hauptprogramm ---------------------------------*)

PROCEDURE SchuelerEingabe() : Schueler =
(* Eingabe eines Schuelers durch Eingabe der einzelnen Werte *)

VAR schueler : Schueler;
  BEGIN
  SIO.PutLine("Bitte den Namen des Schuelers eingeben: ");
  schueler.Name := SIO.GetLine(); 
  SIO.PutLine("Bitte den Vornamen des Schuelers eingeben: ");
  schueler.Vorname := SIO.GetLine(); 
  SIO.PutLine("Bitte das Alter des Schuelers eingeben: ");
  schueler.Alter := SIO.GetReal(); 
  SIO.PutLine("Bitte die Note des Schuelers eingeben: ");
  schueler.Note := SIO.GetReal(); 
  RETURN schueler;
END SchuelerEingabe;

(* Menue *)

BEGIN
  wahl := 0;
  WHILE wahl # 10 DO
    SIO.PutLine("************************** Menue **************************");
    SIO.PutLine("Waehlen Sie einen Menuepunkt:");
    SIO.Nl();
    SIO.PutLine("Schueler zur Liste hinzufuegen (1)");
    SIO.Nl();
    SIO.PutLine("Schuelerliste ausgeben (2)");
    SIO.Nl();
    SIO.PutLine("Schueler ueber einem einzugebenden Alter ausgeben (3)");
    SIO.Nl();
    SIO.PutLine("Schueler schlechter als eine einzugebende Note ausgeben (4)");
    SIO.Nl();
    SIO.PutLine("Schueler besser als eine einzugebende Note ausgeben (5)," );
    SIO.Nl();
    SIO.PutLine("Schueler ueber einem einzugebenden Alter und schlechter");
    SIO.PutLine("schlechter als eine einzugebenden Note ausgeben (6)"); 
    SIO.Nl();
    SIO.PutLine("Schueler ueber einem einzugebenden Alter und besser");
    SIO.PutLine("als eine einzugebende Note ausgeben (7)"); 
    SIO.Nl();
    SIO.PutLine("Arithmetisches Mittel der Noten berechnen (8)" ); 
    SIO.Nl();
    SIO.PutLine("Geometrisches Mittel der Noten berechnen (9)" );
    SIO.Nl();
    SIO.PutLine("Programm beenden (10)");
    SIO.PutLine("************************** Ende Menue **************************");
    SIO.Nl();
    SIO.PutText("Wahl: ");
    wahl := SIO.GetInt();
    dummy := SIO.GetLine(); 
    IF wahl = 1 THEN 
      schueler := SchuelerEingabe();
      Einfuegen(schuelerListe, schueler);
    ELSIF wahl = 2 THEN 
      Ausgeben(schuelerListe);
    ELSIF wahl = 3 THEN
      sortiert := NIL;
      SIO.PutLine("Bitte die Altersgrenze eingeben: ");
      limitAlter := SIO.GetReal();
      dummy := SIO.GetLine(); 
      SIO.Nl();
      SIO.PutLine("Schueler ueber ");
      SIO.PutReal(limitAlter);
      SIO.PutLine(" (aufsteigend sortiert nach Alter):");
      SIO.Nl();
      FilterAlter(schuelerListe, limitAlter, sortiert);
      Ausgeben(sortiert);

     ELSIF wahl = 4 THEN
      sortiert := NIL;
      SIO.PutLine("Bitte die Notengrenze eingeben: ");
      limitNote := SIO.GetReal();
      dummy := SIO.GetLine(); 
      SIO.Nl();
      SIO.PutText("Schueler mit Note ueber ");
      SIO.PutReal(limitNote);
      SIO.PutLine(" (sortiert nach Note):");
      SIO.Nl();
      FilterNote_schlechter_spaeter(schuelerListe, limitNote, sortiert);
      Ausgeben(sortiert);
 
    ELSIF wahl = 5 THEN
      sortiert := NIL;
      SIO.PutLine("Bitte die Notengrenze eingeben: ");
      limitNote := SIO.GetReal();
      dummy := SIO.GetLine(); 
      SIO.Nl();
      SIO.PutText("Schueler mit Note unter ");
      SIO.PutReal(limitNote);
      SIO.PutLine(" (sortiert nach Note):");
      SIO.Nl();
      FilterNote_besser_spaeter(schuelerListe, limitNote, sortiert);
      Ausgeben(sortiert);
   
   ELSIF wahl = 6 THEN
      sortiert := NIL;
      SIO.PutLine("Bitte die Altersgrenze eingeben: ");
      limitAlter := SIO.GetReal();
      dummy := SIO.GetLine(); 
      SIO.PutLine("Bitte die Notengrenze eingeben: ");
      limitNote := SIO.GetReal();
      dummy := SIO.GetLine(); 
      SIO.Nl();
      SIO.PutText("Schueler ueber ");
      SIO.PutReal(limitAlter);
      SIO.PutLine(" und Note ueber ");
      SIO.PutReal(limitNote);
      SIO.PutLine(" (sortiert nach Note):");
      SIO.Nl();
      Filtern_schlechter_spaeter(schuelerListe, limitAlter, limitNote, sortiert);
      Ausgeben(sortiert); 

   ELSIF wahl = 7 THEN
      sortiert := NIL;
      SIO.PutLine("Bitte die Altersgrenze eingeben: ");
      limitAlter := SIO.GetReal();
      dummy := SIO.GetLine(); 
      SIO.PutLine("Bitte die Notengrenze eingeben: ");
      limitNote := SIO.GetReal();
      dummy := SIO.GetLine(); 
      SIO.Nl();
      SIO.PutText("Schueler ueber ");
      SIO.PutReal(limitAlter);
      SIO.PutLine(" und Note unter ");
      SIO.PutReal(limitNote);
      SIO.PutLine(" (sortiert nach Note):");
      SIO.Nl();
      Filtern_besser_spaeter(schuelerListe, limitAlter, limitNote, sortiert);
      Ausgeben(sortiert); 
      
    ELSIF wahl = 8 THEN
      SIO.Nl();
      SIO.Nl();
      IF schuelerListe = NIL THEN
        SIO.PutLine("Die Liste ist leer!");
      ELSE
        SIO.PutText("Arithmetisches Mittel der Noten: ");
        SIO.PutReal(Arithm_Mittel(schuelerListe));
      END;
      SIO.Nl();
     dummy := SIO.GetLine(); 
    ELSIF wahl = 9 THEN
      SIO.Nl();
      SIO.Nl();
      IF schuelerListe = NIL THEN
        SIO.PutLine("Die Liste ist leer!");
      ELSE
        SIO.PutText("Geometrisches Mittel der Noten: ");
        SIO.PutReal(Geom_Mittel(schuelerListe));
      END;
      SIO.Nl();
      dummy := SIO.GetLine(); 
   ELSIF wahl # 10 THEN
      SIO.PutLine("Unzulaessige Eingabe! Bitte neu waehlen!");
      SIO.Nl();
    END;
  END;
SIO.PutLine("Programmende!");
END Schuelerliste.
